--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

require "src/lib"
require "src/entities/entities.lua"
require "src/game"

function love.load()

	-- Setup
	config = lib.getConfig()
	media = {}
	love.graphics.setColorMode("replace")
	TLfres.setScreen(config.displaymode, config.screenwidth)
	if not config.music then lib.music:mute() end
	menu:load()

	-- Create global game variable
	game = Game:new()

end

function love.draw()
	TLfres.transform()
	game:draw()
	menu:draw()
end

function love.update(dt)
	TLbind:update(dt)
	game:update(dt)
	menu:update(dt)
	if dt > 0 then -- avoids a very aggressive assert
		tween.update(dt)
	end
	lib.music:update(dt)
end

function love.mousepressed(x, y, button)
	x = x * game.scale
	y = y * game.scale
	game:mousepressed(x, y, button)
	menu:mousepressed(x, y, button)
end

function love.mousereleased(x, y, button)
	x = x * game.scale
	y = y * game.scale
	game:mousereleased(x, y, button)
	menu:mousereleased(x, y, button)
end

function love.keypressed(key)
	if key == "f4" and (love.keyboard.isDown("ralt") or love.keyboard.isDown("lalt")) then
		love.event.push("q")
	end
end

function control.controlPressed(c)
	game:controlPressed(c)
end
