--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

GuardEntity = MovingEntity:subclass("GuardEntity")

-- Variables
GuardEntity.direction = 0	-- Direction (1 = up, 2 = right, 3 = down, 4 = left)

-- Move the guard in accordance to the passed level
function GuardEntity:move(level)

	-- Check next gridcell
	local nextPosX = self.x
	local nextPosY = self.y
	if self.direction == 1 then
		nextPosY = nextPosY - 1
	elseif self.direction == 2 then
		nextPosX = nextPosX + 1
	elseif self.direction == 3 then
		nextPosY = nextPosY + 1
	elseif self.direction == 4 then
		nextPosX = nextPosX - 1
	end
	
	if level:isOpen(nextPosX, nextPosY) then
		-- Target next position
		self:setTarget(nextPosX - self.x, nextPosY - self.y)
	else
		-- Turn around
		if self.direction == 1 then
			self.direction = 3
		elseif self.direction == 2 then
			self.direction = 4
		elseif self.direction == 3 then
			self.direction = 1
		elseif self.direction == 4 then
			self.direction = 2
		end
		
		-- Try again
		self:move(level)
	end

end

-- Check whether the guard can see the specified position
function GuardEntity:canSee(level, x, y)
	if self.x == x and self.y == y then return true
	elseif level:isOpen(x, y) == false then return false end

	-- Build a grid of positions in the "view cone"
	local grid = {
		left = {},
		front = {},
		right = {}
	}

	grid.front[1] = {x = 0, y = 0} -- to not break the loops below

	if self.direction == 1 then -- up
		grid.left[1] = {x = -1, y = 0}
		grid.left[2] = {x = -1, y = -1}
		grid.left[3] = {x = -1, y = -2}
		grid.front[2] = {x = 0, y = -1}
		grid.front[3] = {x = 0, y = -2}
		grid.right[1] = {x = 1, y = 0}
		grid.right[2] = {x = 1, y = -1}
		grid.right[3] = {x = 1, y = -2}
	elseif self.direction == 2 then -- right
		grid.left[1] = {x = 0, y = -1}
		grid.left[2] = {x = 1, y = -1}
		grid.left[3] = {x = 2, y = -1}
		grid.front[2] = {x = 1, y = 0}
		grid.front[3] = {x = 2, y = 0}
		grid.right[1] = {x = 0, y = 1}
		grid.right[2] = {x = 1, y = 1}
		grid.right[3] = {x = 2, y = 1}
	elseif self.direction == 3 then -- down
		grid.left[1] = {x = 1, y = 0}
		grid.left[2] = {x = 1, y = 1}
		grid.left[3] = {x = 1, y = 2}
		grid.front[2] = {x = 0, y = 1}
		grid.front[3] = {x = 0, y = 2}
		grid.right[1] = {x = -1, y = 0}
		grid.right[2] = {x = -1, y = 1}
		grid.right[3] = {x = -1, y = 2}
	elseif self.direction == 4 then -- left
		grid.left[1] = {x = 0, y = 1}
		grid.left[2] = {x = -1, y = 1}
		grid.left[3] = {x = -2, y = 1}
		grid.front[2] = {x = -1, y = 0}
		grid.front[3] = {x = -2, y = 0}
		grid.right[1] = {x = 0, y = -1}
		grid.right[2] = {x = -1, y = -1}
		grid.right[3] = {x = -2, y = -1}
	end

	-- This blocks everything
	if level:isOpen(
		self.x + grid.front[2].x,
		self.y + grid.front[2].y) == false then
		return false
	end

	-- Ensure that the positions are visible and not blocked by walls
	for i1,direction in pairs(grid) do
		for i = table.getn(direction)-1,1,-1 do
			if level:isOpen(
				self.x + direction[i].x,
				self.y + direction[i].y) == false then
				direction[i+1] = nil
			end
		end
		-- Remove the periph
		direction[1] = nil
	end

	-- Check if the positions can be seen
	for i1,direction in pairs(grid) do
		for i2,position in pairs(direction) do
			if position ~= nil and
				self.x + position.x == x and
				self.y + position.y == y then
				return true
			end
		end
	end
	
	return false
end

-- Overwritten from Entity

function GuardEntity:draw()
	love.graphics.push()
	
	-- Draw the "cone"
	local x1 = self.realX + (game.gridsize / 2)
	local y1 = self.realY + (game.gridsize / 2)
	local x2 = x1
	local y2 = y1
	local x3 = x1
	local y3 = y1
	
	if self.direction == 1 then -- up
		x2 = x1 - game.gridsize
		y2 = y1 - (game.gridsize * 2)
		x3 = x1 + game.gridsize
		y3 = y1 - (game.gridsize * 2)
	elseif self.direction == 2 then -- right
		x2 = x1 + (game.gridsize * 2)
		y2 = y1 - game.gridsize
		x3 = x1 + (game.gridsize * 2)
		y3 = y1 + game.gridsize
	elseif self.direction == 3 then -- down
		x2 = x1 - game.gridsize
		y2 = y1 + (game.gridsize * 2)
		x3 = x1 + game.gridsize
		y3 = y1 + (game.gridsize * 2)
	elseif self.direction == 4 then -- left
		x2 = x1 - (game.gridsize * 2)
		y2 = y1 + game.gridsize
		x3 = x1 - (game.gridsize * 2)
		y3 = y1 - game.gridsize
	end
	
	love.graphics.setColor(config.colors.guardcone)
	love.graphics.triangle("fill", x1, y1, x2, y2, x3, y3)
	
	-- Draw the guard
	love.graphics.draw(media.images.guard, self.realX, self.realY, 0, self.scale, self.scale)

	love.graphics.pop()
end
