--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

LevelEntity = Entity:subclass("LevelEntity")

-- Variables
LevelEntity.scale = 0	-- Scale of the map (relative to screen size)
LevelEntity.map = nil	-- The map to draw
LevelEntity.exit = nil	-- The exit

-- Init
function LevelEntity:initialize(level)
	self.map = level.map
	self.exit = level.exit

	-- Calculate the level area
	self.width = table.getn(self.map) * game.gridsize
	self.height = table.getn(self.map[1]) * game.gridsize
	self.x = ((love.graphics.getWidth() * game.scale) - self.width) / 2
	self.y = ((love.graphics.getHeight() * game.scale) - self.height) / 2
	self.scale = game.gridsize / media.images.floor:getWidth()
end

-- Check whether the specified grid cell is open
function LevelEntity:isOpen(x, y)
	if x < 1 or y < 1 then
		return false
	elseif x > table.getn(self.map) or y > table.getn(self.map[1]) then
		return false
	elseif self.map[x][y] == 1 then
		return false
	end
	
	return true
end

-- Translate a grid position to absolute x,y coordinates
function LevelEntity:translatePosition(x, y)
	x = self.x + ((x - 1) * game.gridsize)
	y = self.y + ((y - 1) * game.gridsize)
	return { x = x, y = y }
end

-- Get the grid positions of absolute coordinates
function LevelEntity:reverseTranslatePosition(x, y)
	x = x - self.x
	y = y - self.y
	
	if x > 0 and x < self.width and y > 0 and y < self.height then
		return {
			x = math.ceil( x / game.gridsize ),
			y = math.ceil( y / game.gridsize )
		}
	else
		return nil
	end
end

-- Create an empty level
function LevelEntity.createLevel(size)
	local level = {
		map = {},
		exit = {x=size,y=size},
		player = {x=1,y=1},
		guards = {}
	}
	
	for x = 1,size do
		level.map[x] = {}
		for y = 1,size do
			level.map[x][y] = 0
		end
	end
	
	return level
end

-- Overwritten from Entity

function LevelEntity:draw()
	love.graphics.push()
	
	-- Floor/wall tiles
	for x = 1,table.getn(self.map) do
		for y = 1,table.getn(self.map[x]) do
			local posx = (x - 1) * game.gridsize
			local posy = (y - 1) * game.gridsize
			if self.map[x][y] == 1 then
				love.graphics.draw(media.images.wall, posx, posy, 0, self.scale, self.scale)
			else
				love.graphics.draw(media.images.floor, posx, posy, 0, self.scale, self.scale)
			end
		end
	end
	
	-- Exit
	if self.exit.x ~= 0 and self.exit.y ~= 0 then
		local posx = (self.exit.x - 1) * game.gridsize
		local posy = (self.exit.y - 1) * game.gridsize
		love.graphics.draw(media.images.exit, posx, posy, 0, self.scale, self.scale)
	end
	
	-- Grid
	love.graphics.setColor(config.colors.border)
	love.graphics.rectangle('line', -1, -1, self.width + 2, self.height + 2)
	for x=1,table.getn(self.map)-1 do
		love.graphics.line(
			game.gridsize * x, 0,
			game.gridsize * x, self.height)
	end
	for y=1,table.getn(self.map[1])-1 do
		love.graphics.line(
			0, game.gridsize * y,
			self.width - 1, game.gridsize * y)
	end
	
	love.graphics.pop()
end
