--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

MovingEntity = Entity:subclass("MovingEntity")

-- Variables
MovingEntity.scale = 0				-- Scale (for graphics)
MovingEntity.ready = true			-- Whether the entity is ready for instructions
MovingEntity.realX = 0				-- Actual (px) x-position
MovingEntity.realY = 0				-- Actual (px) y-position


-- Set the entity target position (grid coordinates relative to current)
function MovingEntity:setTarget(x, y)
	self.ready = false
	self.x = self.x + x
	self.y = self.y + y
end

-- Overwritten from Entity

function MovingEntity:initialize(x, y)
	Entity.initialize(self, x, y)

	self.realX = (x - 1) * game.gridsize
	self.realY = (y - 1) * game.gridsize
	self.scale = game.gridsize / media.images.player:getWidth()
end

function MovingEntity:update(dt)
	
	-- Always move to grid position
	local targetX = (self.x - 1) * game.gridsize
	local targetY = (self.y - 1) * game.gridsize
	if self.realX ~= targetX or self.realY ~= targetY then
		
		-- Calculate target X position
		if self.realX ~= targetX then
			if self.realX > targetX then
				targetX = self.realX - (config.speed * dt)
				if targetX < (self.x - 1) * game.gridsize then
					targetX = (self.x - 1) * game.gridsize
				end
			else
				targetX = self.realX + (config.speed * dt)
				if targetX > (self.x - 1) * game.gridsize then
					targetX = (self.x - 1) * game.gridsize
				end
			end
		end
		
		-- Calculate target Y position
		if self.realY ~= targetY then
			if self.realY > targetY then
				targetY = self.realY - (config.speed * dt)
				if targetY < (self.y - 1) * game.gridsize then
					targetY = (self.y - 1) * game.gridsize
				end
			else
				targetY = self.realY + (config.speed * dt)
				if targetY > (self.y - 1) * game.gridsize then
					targetY = (self.y - 1) * game.gridsize
				end
			end
		end
		
		self.realX = targetX
		self.realY = targetY
		
		if self.realX == (self.x - 1) * game.gridsize and self.realY == (self.y - 1) * game.gridsize then
			self.ready = true
		end
	end
	
end
