--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

lib = {} -- holds all the helper libs

-- A*
--require "src/lib/astar"

-- MiddleClass
require "src/libraries/middleclass/middleclass"
Stateful = require "src/libraries/middleclass/stateful"

-- TLbind
TLbind,control = love.filesystem.load("src/libraries/TLbind/TLbind.lua")()

-- TLfres
require "src/libraries/TLfres/TLfres"

-- Menu
menu = require "src/libraries/menu/menu"

-- Tween
tween = require "src/libraries/tween/tween"

-- A*
Astar = require "src/libraries/LuAstar/LuAstar"

-- Music
lib.music = require "src/libraries/music.lua"

-- Get a path
function lib.astar(map, startpos, endpos, allowdiagonal, relative)
	-- Standard params
	if allowdiagonal == nil then allowdiagonal = false end
	if relative == nil then relative = true end

	-- Setup pathfinding
	Astar(map)
	Astar:setObstValue(1)
	Astar:setInitialNode(startpos.y, startpos.x) -- the way I traverse the map is opposite of his
	Astar:setFinalNode(endpos.y, endpos.x)
	if allowdiagonal then
		Astar:enableDiagonalMove()
	else
		Astar:disableDiagonalMove()
	end

	-- Go get 'em
	local path = Astar:getPath()
	if not path then return nil end

	local instructions = {}

	if relative then
		-- Change instructions to relative
		local prev = table.remove(path, 1) -- starting node
		
		while table.getn(path) ~= 0 do
			local current = table.remove( path, 1 )
			if current.x ~= prev.x or current.y ~= prev.y then
				table.insert(instructions, {
					x = ( current.y - prev.y ),
					y = ( current.x - prev.x )
				})
				prev = current
			end
		end
	else
		-- Reverse the instructions due to differences in how the map is traversed
		local prev = table.remove(path, 1) -- remove starting node
		
		while table.getn(path) ~= 0 do
			local current = table.remove( path, 1 )
			if current.x ~= prev.x or current.y ~= prev.y then
				table.insert(instructions, {
					x = current.y,
					y = current.x
				})
				prev = current
			end
		end
	end

	return instructions
end

-- Get a list of the files in a folder
function lib.listFiles(path)
	local files = {}

	for i,v in ipairs(love.filesystem.enumerate(path)) do
		if v:sub(0,1) ~= "." then
			local name = v:sub(0,-5)
			files[name] = path .. "/" .. v
		end
	end

	return files
end

-- Loads the options
function lib.loadObject(path)
	local chunk = love.filesystem.load(path)
	return chunk()
end

-- Saves the options
function lib.saveObject(path, options)
	local text = "return " .. lib.serialize(options)
	lib.makePath(path)
	return love.filesystem.write(path, text)
end

-- Recursively creates the required path
function lib.makePath(path)
	path = path:split("/")
	local fullpath = ""
	for i = 1,(#path-1) do
		fullpath = fullpath .. path[i] .. "/"
		if not love.filesystem.isDirectory(fullpath) then
			love.filesystem.mkdir(fullpath)
		end
	end
end

-- Split a string
function string:split(sep)
	local sep, fields = sep or ":", {}
	local pattern = string.format("([^%s]+)", sep)
	self:gsub(pattern, function(c) fields[#fields+1] = c end)
	return fields
end

-- Variable to string serialization
function lib.serialize(var)
	local string = ""
	
	if type(var) == "boolean" then
		if var then
			string = "true"
		else
			string = "false"
		end
	elseif type(var) == "string" then
		string = "\"" .. var .. "\""
	elseif type(var) == "table" then
		string = "{\n"
		
		for k,v in pairs(var) do
			if type(k) == "string" then
				string = string .. "['" .. k .. "'] = " .. lib.serialize(v) .. ",\n"
			else
				string = string .. lib.serialize(v) .. ",\n"
			end
		end
		
		string = string .. "}"
	else
		string = var
	end
	
	return string
end

-- Builds the menus
function lib.buildMenus()
	local menus = lib.listFiles("src/menus")
	
	for k,v in pairs(menus) do
		menus[k] = require(v)
	end

	return menus
end

-- Loads a level
function lib.loadLevel(level)

	chunk = love.filesystem.load(level) -- load the chunk
	return chunk() -- execute the chunk

end

-- Gets a specific level
function lib.getLevel(i)
	
	local levels = lib.listLevels()
	return levels[i]
	
end

-- Crafts a new level
function lib.newLevel(w,h)
	local level = {
		-- Name of the map
		name = "",
		-- Author
		author = "",
		-- 2D map matrix
		map = {
		},
		-- GET THE FUCK OUT
		exit = { x = 0, y = 0 },
		-- Player start location
		player = { x = 0, y = 0 },
		-- Guards
		guards = {
		}
	}

	for i = 1,w do
		local row = {}
		for j = 1,h do
			table.insert(row, 0)
		end
		table.insert(level.map, row)
	end

	return level
end

-- Gets a list of the levels
function lib.listLevels(path)

	local levels = {}
	
	for i,v in ipairs(love.filesystem.enumerate("levels/" .. path)) do
		if v:sub(0,1) ~= "." then
			table.insert(levels, "levels/" .. path .. "/" .. v)
		end
	end
	
	return levels
	
end

-- Gets the config
function lib.getConfig()
	return lib.loadObject("config.lua")
end

-- Get the mouse position (relative to game scale)
function lib.getMousePosition()
	return (love.mouse.getX() * game.scale), (love.mouse.getY() * game.scale)
end
