--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local MenuItem = require "src/libraries/menu/item/item"
local Button = MenuItem:subclass("Button")

-- Initialize
function Button:initialize(x, y, parent)
	MenuItem.initialize(self, x, y, parent)

	-- Whether the button is being hovered over
	self.hover = false

	-- Whether the button is being pressed
	self.press = false
end

-- Update
function Button:update(dt)
	-- Check if the mouse is inside the button
	local x,y = lib.getMousePosition()
	local buttonInside = self:hasInside(x, y, true)

	if buttonInside == true and self.hover == false then
		self.hover = true
		self:triggerEvent("hover", {})
	elseif buttonInside == false and self.hover == true then
		self.hover = false
		self:triggerEvent("leave", {})
	end
end

-- Mouse events
function Button:mousepressed(x, y, button)
	if self:hasInside(x, y) and self.press == false then
		self.press = true
		self:triggerEvent("press", {x = x, y = y, button = button})
	end
end

function Button:mousereleased(x, y, button)
	local hasInside = self:hasInside(x, y)

	if hasInside and self.press == true then
		self:action()
	end

	if hasInside then
		self:triggerEvent("release", {x = x, y = y, button = button})
	end
	if self.press == true then
		self.press = false
	end
end

-- Is selectable
function Button:isSelectable() return self:isActive() end

-- Action
function Button:action()
	self:triggerEvent("click", {x = x, y = y, button = button})
end

-- Check if the position is inside the button
function Button:hasInside(x, y, absolute)
	if absolute then
		x = x - self:getAbsoluteX()
		y = y - self:getAbsoluteY()
	else
		x = x - self:getX()
		y = y - self:getY()
	end
	if x >= 0 and x <= self:getWidth() and y >= 0 and y <= self:getHeight() then
		return true
	else
		return false
	end
end

return Button
