--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local Button = require "src/libraries/menu/item/button"
local ImageButton = Button:subclass("ImageButton")

-- Initialize
function ImageButton:initialize(x, y, parent)
	Button.initialize(self, x, y, parent)

	-- Images
	self.images = {
		default = nil,
		hover = nil,
		press = nil
	}
end

-- Get/set
function ImageButton:getImages() return self.images end
function ImageButton:setImages(i) self.images = i end
function ImageButton:getDefaultImage() return self.images.default end
function ImageButton:setDefaultImage(i) self.images.default = i end
function ImageButton:getHoverImage() return self.images.hover end
function ImageButton:setHoverImage(i) self.images.hover = i end
function ImageButton:getPressImage() return self.images.press end
function ImageButton:setPressImage(i) self.images.press = i end

-- Size
function ImageButton:getWidth()
	if self.width ~= 0 then return self.width end

	if self.press and self.hover and self.images.press then
		return self.images.press:getWidth()
	elseif self.hover and self.images.hover then
		return self.images.hover:getWidth()
	elseif self.images.default then
		return self.images.default:getWidth()
	else
		return 0
	end
end
function ImageButton:getHeight()
	if self.height ~= 0 then return self.height end
	
	if self.press and self.hover and self.images.press then
		return self.images.press:getHeight()
	elseif self.hover and self.images.hover then
		return self.images.hover:getHeight()
	elseif self.images.default then
		return self.images.default:getHeight()
	else
		return 0
	end
end

-- Draw
function ImageButton:draw()
	local image = nil
	local y = 0

	if self.press and self.hover and self.images.press then
		image = self.images.press
	elseif (self.hover or self.selected) and self.images.hover then
		image = self.images.hover
	elseif self.images.default then
		image = self.images.default
	end

	if self.press and self.hover then y = 2 end

	if image then
		love.graphics.draw(image, self.x, self.y + y)
	end
end

return ImageButton
