--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local MenuItem = class("MenuItem")

-- Initialize
function MenuItem:initialize(x, y, parent)
	-- Position
	self.x = x
	self.y = y

	-- Size
	self.width = 0
	self.height = 0

	-- Visibility
	self.visible = true

	-- Activity
	self.active = true

	-- Selected
	self.selected = false

	-- Parent
	self.parent = parent

	-- Event listeners
	self.eventListeners = {}

	-- Span the entire column (used in grid layout)
	self.colspan = false
end

-- Position
function MenuItem:getPosition() return self:getX(), self:getY() end
function MenuItem:getX() return self.x end
function MenuItem:getY() return self.y end
function MenuItem:setPosition(x, y) self:setX(x) self:setY(y) end
function MenuItem:setX(x) self.x = x end
function MenuItem:setY(y) self.y = y end
function MenuItem:getAbsolutePosition() return self:getAbsoluteX(), self:getAbsoluteY() end
function MenuItem:getAbsoluteX()
	local x = self:getX()
	if self.parent then
		x = x + self.parent:getAbsoluteX()
	end
	return x
end
function MenuItem:getAbsoluteY()
	local y = self:getY()
	if self.parent then
		y = y + self.parent:getAbsoluteY()
	end
	return y
end

-- Get the size
function MenuItem:getSize() return self:getWidth(), self:getHeight() end
function MenuItem:getWidth() return self.width end
function MenuItem:getHeight() return self.height end
function MenuItem:setSize(w, h) self:setWidth(w) self:setHeight(h) end
function MenuItem:setWidth(w) self.width = w end
function MenuItem:setHeight(h) self.height = h end

-- Visibility
function MenuItem:isVisible() return self.visible end
function MenuItem:setVisibility(v)
	self.visible = v
	if v then
		self:triggerEvent("show", {})
	else
		self:triggerEvent("hide", {})
	end
end
function MenuItem:show() self:setVisibility(true) end
function MenuItem:hide() self:setVisibility(false) end

-- Active
function MenuItem:isActive() return self.active end
function MenuItem:setActive(a) self.active = a end
function MenuItem:activate() self:setActive(true) end
function MenuItem:deactivate() self:setActive(false) end

-- Enable/disable
function MenuItem:enable() self:activate() self:show() end
function MenuItem:disable() self:deactivate() self:hide() end

-- Select
function MenuItem:isSelectable() return false end
function MenuItem:setSelected(s) self.selected = s if s then self:triggerEvent("selected", {}) end end
function MenuItem:select() self:setSelected(true) end
function MenuItem:deselect() self:setSelected(false) end

-- Perform an action
function MenuItem:action() end

-- Internal events
function MenuItem:addEventListener(event, func)
	if self.eventListeners[event] == nil then
		self.eventListeners[event] = {}
	end
	table.insert(self.eventListeners[event], func)
end
function MenuItem:clearEventListeners(event)
	self.eventListeners[event] = nil
end
function MenuItem:triggerEvent(event, params)
	if self.eventListeners[event] == nil then return end
	for i,v in ipairs(self.eventListeners[event]) do
		v(event, self, params)
	end
end

-- Refresh
function MenuItem:refresh() end

-- Draw the item
function MenuItem:draw() end

-- Update the item
function MenuItem:update(dt) end

-- Handle mouse events
function MenuItem:mousepressed(x, y, button) end
function MenuItem:mousereleased(x, y, button) end

-- Handle keyboard events
function MenuItem:keypressed(key) end
function MenuItem:keyreleased(key) end

return MenuItem
