--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local MenuItem = require "src/libraries/menu/item/item"
local Label = MenuItem:subclass("Label")

-- Initialize
function Label:initialize(x, y, parent)
	MenuItem.initialize(self, x, y, parent)

	-- The text for the label
	self.text = ""

	-- Font/color to display the text in
	self.font = nil
	self.color = nil
end

-- Get/set
function Label:getText() return self.text end
function Label:setText(t) self.text = t end
function Label:getFont() return self.font end
function Label:setFont(f) self.font = f end
function Label:getColor() return self.color end
function Label:setColor(c) self.color = c end

-- Size
function Label:getWidth()
	if self.width ~= 0 then return self.width end

	if self.font then
		return self.font:getWidth(self.text)
	else
		return self.width
	end
end

function Label:getHeight()
	if self.height ~= 0 then return self.height end

	if self.font then
		return self.font:getHeight()
	else
		return self.height
	end
end

-- Draw
function Label:draw()
	love.graphics.push()
	love.graphics.setColorMode("modulate")

	if self.color then love.graphics.setColor(self.color) end
	if self.font then love.graphics.setFont(self.font) end
	
	love.graphics.print(self.text, self.x, self.y)

	love.graphics.setColorMode("replace")
	love.graphics.pop()
end

return Label
