--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local Button = require "src/libraries/menu/item/button"
local ScaleButton = Button:subclass("ScaleButton")

-- Initialize
function ScaleButton:initialize(x, y, parent)
	Button.initialize(self, x, y, parent)

	-- Image
	self.image = nil

	-- Scale
	self.scale = 1.25
end

-- Get/set
function ScaleButton:getImage() return self.image end
function ScaleButton:setImage(i) self.image = i end
function ScaleButton:getScale() return self.scale end
function ScaleButton:setScale(s) self.scale = s end

-- Size
function ScaleButton:getWidth()
	if self.width ~= 0 then return self.width end

	if self.image then
		return self.image:getWidth()
	else
		return 0
	end
end
function ScaleButton:getHeight()
	if self.height ~= 0 then return self.height end
	
	if self.image then
		return self.image:getHeight()
	else
		return 0
	end
end

-- Draw
function ScaleButton:draw()
	local s = 1
	local x = self:getX()
	local y = self:getY()

	if self.press and self.hover then
		y = y + 2
	end
	if (self.hover or self.selected) then
		s = self.scale
		local w = self:getWidth()
		local h = self:getHeight()
		x = x + ((w - (w * s)) / 2)
		y = y + ((h - (h * s)) / 2)
	end

	if self.image then
		love.graphics.draw(self.image, x, y, 0, s, s)
	end
end

return ScaleButton
