--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local TextButton = require "src/libraries/menu/item/textbutton"
local Selection = TextButton:subclass("Selection")

-- Initialize
function Selection:initialize(x, y, parent)
	TextButton.initialize(self, x, y, parent)

	-- The possible options
	self.options = {}

	-- The currently selected option
	self.current = 0
end

-- Get/set
function Selection:getOptions() return self.options end
function Selection:getOption(i) return self.options[i] end
function Selection:setOptions(o) self.options = o if self.current == 0 then self:setCurrent(1) end self:updateText() end
function Selection:addOption(o) table.insert(self.options, o) if self.current == 0 then self:setCurrent(1) end self:updateText() end
function Selection:getCurrent() return self.current end
function Selection:getCurrentOption() return self.options[self.current] end
function Selection:setCurrent(i)
	self.current = i
	self:updateText()
	self:triggerEvent("change", { current = self.current })
end
function Selection:setCurrentOption(o) for i,v in ipairs(self.options) do if v == o then self:setCurrent(i) return end end end

-- Update text
function Selection:updateText()
	if table.getn(self.options) == 0 or self.current == 0 then return end
	self.text = self.options[self.current]
end

-- Action
function Selection:action()
	if table.getn(self.options) == 0 then return end

	local current = self.current + 1
	if current > table.getn(self.options) then
		current = 1
	end

	self:setCurrent(current)
end

return Selection
