--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local Button = require "src/libraries/menu/item/button"
local TextButton = Button:subclass("TextButton")

-- Initialize
function TextButton:initialize(x, y, parent)
	Button.initialize(self, x, y, parent)

	-- Text to display on the button
	self.text = ""

	-- Font/color to draw button text
	self.colors = {
		default = nil,
		hover = nil,
		press = nil
	}
	self.font = nil
end

-- Get/set
function TextButton:getText() return self.text end
function TextButton:setText(t) self.text = t end
function TextButton:getColors() return self.colors end
function TextButton:setColors(c) self.colors = c end
function TextButton:getDefaultColor() return self.colors.default end
function TextButton:setDefaultColor(c) self.colors.default = c end
function TextButton:getHoverColor() return self.colors.hover end
function TextButton:setHoverColor(c) self.colors.hover = c end
function TextButton:getPressColor() return self.colors.press end
function TextButton:setPressColor(c) self.colors.press = c end
function TextButton:getFont() return self.font end
function TextButton:setFont(f) self.font = f end

-- Size
function TextButton:getWidth()
	if self.width ~= 0 or self.text == "" then return self.width end
	return self:getFont():getWidth(self.text)
end
function TextButton:getHeight()
	if self.height ~= 0 or self.text == "" then return self.height end
	return self:getFont():getHeight()
end

-- Draw
function TextButton:draw()
	love.graphics.push()
	love.graphics.setColorMode("modulate")

	if self.font then love.graphics.setFont(self.font) end
	if self.press and self.hover and self.colors.press then
		love.graphics.setColor(self.colors.press)
	elseif (self.hover or self.selected) and self.colors.hover then
		love.graphics.setColor(self.colors.hover)
	elseif self.colors.default then
		love.graphics.setColor(self.colors.default)
	end

	if self.press and self.hover then
		love.graphics.print(self.text, self.x, self.y + 2)
	else
		love.graphics.print(self.text, self.x, self.y)
	end

	love.graphics.setColorMode("replace")	
	love.graphics.pop()
end

return TextButton
