--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local menu = {}

-- List of children
menu.children = {}

-- List of tagged items
menu.tags = {}

-- Load the menu system
function menu:load()
end

-- Draw the menus
function menu:draw()
	for i,v in ipairs(self.children) do
		if v.visible then
			v:draw()
		end
	end
end

-- Update the menus
function menu:update(dt)
	for i,v in ipairs(self.children) do
		if v.active and v.visible then
			v:update(dt)
		end
	end
end

-- Handle mouse presses
function menu:mousepressed(x, y, button)
	for i,v in ipairs(self.children) do
		if v.active and v.visible then
			v:mousepressed(x, y, button)
		end
	end
end

function menu:mousereleased(x, y, button)
	for i,v in ipairs(self.children) do
		if v.active and v.visible then
			v:mousereleased(x, y, button)
		end
	end
end

-- Handle keypresses
function menu:keypressed(key)
	local captured = false
	for i,v in ipairs(self.children) do
		if v.active and v.visible then
			captured = v:keypressed(key)
			if captured then return end
		end
	end
end

function menu:keyreleased(key)
	local captured = false
	for i,v in ipairs(self.children) do
		if v.active and v.visible then
			captured = v:keyreleased(key)
			if captured then return end
		end
	end
end

-- Add children
function menu:addPanel(x, y)
	local Panel = require "src/libraries/menu/item/panel"
	local panel = Panel:new(x, y)
	table.insert(self.children, panel)
	return panel
end

-- Remove children
function menu:removePanel(panel)
	for i,v in ipairs(self.children) do
		if panel == v then
			table.remove(self.children, i)
			return
		end
	end
end

-- Tag an item
function menu:tag(item, tag)
	self.tags[tag] = item
end

-- Get an item based on the tag
function menu:get(tag)
	return self.tags[tag]
end

-- Remove an item based on the tag
function menu:removeTag(tag)
	self.tags[tag] = nil
end

-- Clear all tags
function menu:clearTags()
	self.tags = {}
end

return menu