--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

MusicManager = class("MusicManager")

-- Initialize
function MusicManager:initialize()
	-- Currently playing source
	self.current = nil

	-- Queue of music to play
	self.sourceQueue = {}

	-- Whether the music is muted
	self.muted = false
end

-- Play a source
function MusicManager:play(source, time)
	if source then
		if source == self.current then return end
		source:setLooping(true)
		if not time then time = 0 end
		table.insert(self.sourceQueue, { source = source, time = time})
	end
	if not self.muted and self.current then
		self.current:play()
	end
end

-- Mute/unmute the music
function MusicManager:mute()
	if self.muted == true then return end
	self.muted = true
	self:stop()
end

function MusicManager:unmute()
	if self.muted == false then return end
	self.muted = false
	self:play()
end

-- Stop the music
function MusicManager:stop()
	if self.current and not self.current:isStopped() then
		self.current:stop()
	end
end

-- Update the manager
function MusicManager:update(dt)
	for i,v in ipairs(self.sourceQueue) do
		v.time = v.time - dt
		if v.time < 0 then
			self:stop()
			self.current = v.source
			self:play()
			table.remove(self.sourceQueue, i)
			return
		end
	end
end

return MusicManager:new()
