--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local options = menu:addPanel(500, 300)
local displayChanged = false
options:setSize(600, 400)
options:addEventListener("escape", function(event, object, params)
	game:popState("Options")
end)

local label = options:addLabel("Fullscreen")
label:setFont(media.fonts.menu)
label:setColor(media.colors.text)

local fullscreen = options:addCheckbox()
fullscreen:setFont(media.fonts.menu)
fullscreen:setColors(media.colors.button)
fullscreen:addEventListener("toggle", function(event, object, params)
	config.displaymode.full = params.setting
	options:refresh()
	displayChanged = true
end)

local label = options:addLabel("Resolution")
label:setFont(media.fonts.menu)
label:setColor(media.colors.text)

local modes = love.graphics.getModes()
local modesstring = {}
local currentMode = 0
table.sort(modes, function(a, b) return a.width*a.height < b.width*b.height end)
-- Filter out unwanted modes
for i,v in ipairs(modes) do
	if v.width < 600 or v.width > 1600 then
		table.remove(modes, i)
	else
		local str = v.width .. "x" .. v.height
		table.insert(modesstring, str)
	end
end

local resloution = options:addSelection(modesstring)
resloution:setFont(media.fonts.menu)
resloution:setColors(media.colors.button)
resloution:addEventListener("change", function(event, object, params)
	config.displaymode.w = modes[params.current].width
	config.displaymode.h = modes[params.current].height
	options:refresh()
	displayChanged = true
end)

local label = options:addLabel("Sound Effects")
label:setFont(media.fonts.menu)
label:setColor(media.colors.text)

local sound = options:addCheckbox()
sound:setFont(media.fonts.menu)
sound:setColors(media.colors.button)
sound:addEventListener("toggle", function(event, object, params)
	config.sound = params.setting
	options:refresh()
end)

local label = options:addLabel("Music")
label:setFont(media.fonts.menu)
label:setColor(media.colors.text)

local music = options:addCheckbox()
music:setFont(media.fonts.menu)
music:setColors(media.colors.button)
music:addEventListener("toggle", function(event, object, params)
	config.music = params.setting
	options:refresh()
end)

-- Reset the options
local resetOptions = function()
	fullscreen:setSetting(config.displaymode.full)
	for i,v in ipairs(modes) do
		if v.width == config.displaymode.w and v.height == config.displaymode.h then
			resloution:setCurrent(i)
			break
		end
	end
	sound:setSetting(config.sound)
	music:setSetting(config.music)
	displayChanged = false
end

local save = options:addImageButton(media.images.button_save_normal)
save:addEventListener("click", function(event, object, params)
	-- Save settings
	lib.saveObject("config.lua", config)

	-- Apply the visual changes
	if displayChanged then
		TLfres.setScreen(config.displaymode, config.screenwidth)
		game:initialize(true)
		for k,v in pairs(media.menus) do
			v:refresh()
		end
		displayChanged = false
	end

	-- Apply the audible changes
	if config.music then
		lib.music:unmute()
	else
		lib.music:mute()
	end

	game:popState("Options")
end)
save:setHoverImage(media.images.button_save_hover)

local cancel = options:addImageButton(media.images.button_cancel_normal)
cancel:addEventListener("click", function(event, object, params)
	-- Revert changes
	config = lib.loadObject("config.lua")
	resetOptions()
	game:popState("Options")
end)
cancel:setHoverImage(media.images.button_cancel_hover)

options:setLayout("grid", 20)
options:hide()

resetOptions()
return options
