--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local BrowseState = Game:addState("Browse")

function BrowseState:enteredState()
	self:setupMenu(1)
end

function BrowseState:exitedState()
	self.menu:hide()
end

function BrowseState:continuedState()
	self:setupMenu()
	self.menu:show()
	lib.music:play(media.music.menu)
end

function BrowseState:setupMenu(levelGroup)
	-- Setup menu
	local colspan = 5
	local x = 0
	local y = 0
	local w = media.images.player:getWidth()
	local h = media.images.player:getHeight()
	if levelGroup then
		self.currentLevelGroup = levelGroup
	end
	self.levels = lib.listLevels(media.levels[self.currentLevelGroup].path)
	self.menu = media.menus.levels
	self.menu:clearChildren()
	self.menu:setSize(w * colspan, math.ceil(#self.levels / colspan) * h)
	self.menu:setX( ((love.graphics.getWidth() * game.scale) - self.menu:getWidth()) / 2 )
	self.menu:setY( ((love.graphics.getHeight() * game.scale) - self.menu:getHeight()) / 2 )

	-- Levels
	for i,v in ipairs(self.levels) do
		x = (i - 1) % colspan
		y = math.floor((i - 1) / colspan)
		local image = media.images.guard
		if config.score and config.score[v] then
			image = media.images.player
		end
		local button = self.menu:addScaleButton(image, x * w, y * h)
		button:addEventListener("click", function(e,o,p)
			game:play(self.levels[i])
			self:pushState("Game")
		end)
	end

	-- Group navigator
	local groupPrevious = self.menu:addTextButton("<", 0, 0)
	groupPrevious:setFont(media.fonts.menu)
	groupPrevious:setColors({
		default = media.colors.button.default,
		hover = media.colors.button.hover,
		press = nil
	})
	groupPrevious:addEventListener("click", function(e,o,p)
		self:previousLevelGroup()
	end)
	local groupLabel = self.menu:addLabel(media.levels[self.currentLevelGroup].name, 0, 0)
	groupLabel:setFont(media.fonts.menu)
	groupLabel:setColor(media.colors.text)
	local groupNext = self.menu:addTextButton(">", 0, 0)
	groupNext:setFont(media.fonts.menu)
	groupNext:setColors({
		default = media.colors.button.default,
		hover = media.colors.button.hover,
		press = nil
	})
	groupNext:addEventListener("click", function(e,o,p)
		self:nextLevelGroup()
	end)
	groupLabel:setPosition((self.menu:getWidth() - groupLabel:getWidth()) / 2, self.menu:getHeight() + 5)
	groupPrevious:setPosition(groupLabel:getX() - groupPrevious:getWidth() - 20, groupLabel:getY())
	groupNext:setPosition(groupLabel:getX() + groupLabel:getWidth() + 20, groupLabel:getY())

	self.menu:show()

	-- Setup music
	lib.music:play(media.music.menu)
end

function BrowseState:previousLevelGroup()
	local n = self.currentLevelGroup - 1
	if n < 1 then n = #media.levels end
	self:setupMenu(n)
end

function BrowseState:nextLevelGroup()
	local n = self.currentLevelGroup + 1
	if n > #media.levels then n = 1 end
	self:setupMenu(n)
end

function BrowseState:draw()
	-- Background
	local x = ((love.graphics.getWidth() * game.scale) - media.images.background:getWidth()) / 2
	local y = ((love.graphics.getHeight() * game.scale) - media.images.background:getHeight()) / 2
	love.graphics.draw(media.images.background, x, y)

	-- Title
	x = ((love.graphics.getWidth() * game.scale) - media.images.title:getWidth()) / 2
	y = self.menu.y - media.images.title:getHeight()
	love.graphics.draw(media.images.title, x, y)
end

function BrowseState:controlPressed(c)
	if c == "escape" then
		game:popState("Browse")
	end
	self.menu:keypressed(c)
end
