--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local EditorState = Game:addState("Editor")

local level = nil
local editorMode = nil
local entity = {
	level = nil,
	player = nil,
	guards = {}
}
local playerEntity = nil

function EditorState:enteredState()
	self.paused = false
	local selection = menu:get("level-size-selector")
	self:createLevel(selection:getCurrentOption())
	media.menus.editor:show()
	lib.music:play(media.music.menu)
end

function EditorState:continuedState()
	self.paused = false
	media.menus.editor:show()
	lib.music:play(media.music.menu)
end

function EditorState:exitedState()
	media.menus.editor:hide()
	media.menus.editor_pause:hide()
end

function EditorState:draw()
	-- Background
	local x = ((love.graphics.getWidth() * game.scale) - media.images.background:getWidth()) / 2
	local y = ((love.graphics.getHeight() * game.scale) - media.images.background:getHeight()) / 2
	love.graphics.draw(media.images.background, x, y)

	-- Entities
	love.graphics.push()
	love.graphics.translate(entity.level.x, entity.level.y)

	entity.level:draw()
	if entity.player then entity.player:draw() end
	for i,v in ipairs(entity.guards) do
		v:draw()
	end
	
	love.graphics.pop()
end

function EditorState:mousepressed(x, y, button)
	if self.paused then return end

	-- Modify the level
	if (button == 'l') then
		local pos = entity.level:reverseTranslatePosition(x,y)
		if pos then
			if entity.level:isOpen(pos.x, pos.y) or editorMode == "wall" then
				if editorMode == "player" then
					level.player = pos
				elseif editorMode == "guard" then
					local guard = nil
					for i,v in ipairs(level.guards) do
						if v.x == pos.x and v.y == pos.y then
							guard = i
						end
					end
					if guard then
						level.guards[guard].direction = level.guards[guard].direction + 1
						if level.guards[guard].direction > 4 then
							table.remove(level.guards, guard)
						end
					else
						table.insert(level.guards, { x = pos.x, y = pos.y, direction = 1 })
					end
					--@todo
				elseif editorMode == "wall" then
					if level.map[pos.x][pos.y] == 1 then
						level.map[pos.x][pos.y] = 0
					else
						level.map[pos.x][pos.y] = 1
					end
				elseif editorMode == "exit" then
					level.exit = pos
				end

				self:refreshLevel()
			end
			
		end
	end
end

function EditorState:controlPressed(c)
	if c == 'escape' then
		if not self.paused then
			self.paused = true
			media.menus.editor_pause:show()
		else
			self.paused = false
			media.menus.editor_pause:hide()
		end
	end
end

function EditorState:setEditorMode(mode)
	editorMode = mode
end

function EditorState:createLevel(size)
	-- Determine gridsize
	local gridsize = config.gridsize
	local w = (love.graphics.getWidth() * self.scale) / (size + 2)
	local h = (love.graphics.getHeight() * self.scale) / (size + 2)
	if w < gridsize then gridsize = w end
	if h < gridsize then gridsize = h end

	self.gridsize = gridsize

	-- Create level
	level = lib.newLevel(size,size)
	self:refreshLevel()
end

function EditorState:refreshLevel()
	entity.guards = {}
	for i,v in ipairs(level.guards) do
		local guard = GuardEntity:new(v.x, v.y)
		guard.direction = v.direction
		table.insert(entity.guards, guard)
	end
	if level.player.x ~= 0 and level.player.y ~= 0 then
		entity.player = PlayerEntity:new(level.player.x, level.player.y)
	else
		entity.player = nil
	end
	entity.level = LevelEntity:new(level)
end

function EditorState:testLevel()
	local path = "temp.level"
	lib.saveObject(path, level)
	self.levels = {}
	self:play(path)
	self:pushState("Game")
end

function EditorState:saveLevel()
	local path = "user_levels/" .. os.time() .. ".level"
	lib.saveObject(path, level)
	self:popState("Editor")
end

function EditorState:loadLevel(path)
	level = lib.loadLevel(path)
	self:refreshLevel()
end
