--[[
	Ninja vs. Samurai: A sneaky puzzle game
	Created: 12.09.11 by Michael Enger (http://www.thelonelycoder.com)
	Version: 0.3
	Licence: ZLIB
]]

local LoadingState = Game:addState("Loading")

function LoadingState:enteredState()
	-- Enumerate
	media.levels = require "levels/levels.lua"
	media.images = lib.listFiles("images")
	media.audio = lib.listFiles("audio")
	media.music = lib.listFiles("music")
	media.fonts = {}
	media.menus = {}

	-- Load "necessities"
	media.images.background = love.graphics.newImage(media.images.background)
	media.fonts.menu = love.graphics.newFont("fonts/league_gothic.otf", 72)

	-- Load images
	self.thread = love.thread.newThread("images", "src/load/image.lua")
	self.thread:start()

	-- Start with the loading image
	self.loadingImage = "loading"
	self.loadingAudio = nil
	self.loadingMusic = nil
	self.thread:send("path", media.images.loading)
end

function LoadingState:draw()
	if type(media.images.loading) == "string" then return end -- we're still loading the loading image

	-- Background
	local x = ((love.graphics.getWidth() * game.scale) - media.images.background:getWidth()) / 2
	local y = ((love.graphics.getHeight() * game.scale) - media.images.background:getHeight()) / 2
	love.graphics.draw(media.images.background, x, y)

	-- Loading text
	x = ((love.graphics.getWidth() * game.scale) - media.images.loading:getWidth()) / 2
	y = ((love.graphics.getHeight() * game.scale) - media.images.loading:getHeight()) / 2
	love.graphics.draw(media.images.loading, x, y)
end

function LoadingState:update(dt)
	if self.thread ~= nil then
		local error = self.thread:receive("error")
		if error then
			print(error)
			return
		end

		local object = self.thread:receive("object")
		if object then
			if self.loadingImage then
				self.loadingImage = self:objectLoaded(object, self.loadingImage, "images")

				-- Done loading images
				if self.loadingImage == nil then
					self.thread:send("done", "done")

					-- Load audio
					self.thread = love.thread.newThread("audio", "src/load/audio.lua")
					self.thread:start()
					self.loadingAudio = "click"
					self.thread:send("path", media.audio[self.loadingAudio])
				end
			elseif self.loadingAudio then
				self.loadingAudio = self:objectLoaded(object, self.loadingAudio, "audio")

				-- Done loading audio
				if self.loadingAudio == nil then
					self.thread:send("done", "done")
					
					-- Load music
					self.thread = love.thread.newThread("music", "src/load/music.lua")
					self.thread:start()
					self.loadingMusic = "menu"
					self.thread:send("path", media.music[self.loadingMusic])
				end
			elseif self.loadingMusic then
				self.loadingMusic = self:objectLoaded(object, self.loadingMusic, "music")

				-- Done loading audio
				if self.loadingMusic == nil then
					self.thread:send("done", "done")
					self.thread = nil
				end
			end
		end
	else
		-- Setup colors
		media.colors = {
			text = { 95, 95, 85 },
			button = {
				default = { 95, 95, 85 },
				hover = { 52, 52, 46 }
			},
			menu = {
				background = { 157,156,140,150 },
				border = { 255,255,255,40 }
			}
		}

		-- Load menus
		media.menus = lib.buildMenus()

		-- Reset game animations
		game:resetClouds(-100)

		-- We're done here
		self:gotoState("Main")
	end
end

-- Object has been loaded
function LoadingState:objectLoaded(object, name, path)
	local newObject = nil

	if path == "images" then
		media[path][name] = love.graphics.newImage(object)
	else
		media[path][name] = object
	end

	for k,v in pairs(media[path]) do
		if type(media[path][k]) == "string" then
			newObject = k
			self.thread:send("path", media[path][k])
			break
		end
	end

	return newObject
end

